package it.inaf.oats.ia2.tapschemamanager.businesslayer;

import it.inaf.oats.ia2.tapschemamanager.datalayer.SchemaEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.TableEntity;
import it.inaf.oats.ia2.tapschemamanager.datalayer.TapSchemaHandler;
import java.io.Serializable;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class Schema extends EntityWrapper<SchemaEntity> implements EntityWrapperContainer<Table>, Serializable {

    private static final long serialVersionUID = -1564499229970697945L;

    public static final String UTYPE = "utype";
    public static final String DESCRIPTION = "description";

    private TapSchemaHandler tapSchemaHandler;

    private String selectedTable;
    private Map<String, Table> tables;

    public Schema(TapSchemaHandler tapSchemaHandler, SchemaEntity schemaEntity) throws SQLException {
        super(schemaEntity, UTYPE, DESCRIPTION);

        this.tapSchemaHandler = tapSchemaHandler;

        addValue(UTYPE, schemaEntity.getUtype());
        addValue(DESCRIPTION, schemaEntity.getDescription());

        tables = new TreeMap<String, Table>(String.CASE_INSENSITIVE_ORDER);

        for (TableEntity tableEntity : schemaEntity.getTables()) {
            String tableName = tableEntity.getShortTableName();
            tables.put(tableName, new Table(tapSchemaHandler, getName(), tableName, tableEntity));
        }

        setStatus(Status.ADDED_PERSISTED);
    }

    @Override
    protected void afterSetValue(String key, String value) {
        SchemaEntity schemaEntity = getEntity();
        if (key.equals(UTYPE)) {
            schemaEntity.setUtype(value);
        } else if (key.equals(DESCRIPTION)) {
            schemaEntity.setDescription(value);
        }
    }

    @Override
    public List<String> getEntitiesNames() {
        return Util.setToList(tables.keySet());
    }

    @Override
    public List<String> getAddables() {
        return Util.getAddables(tapSchemaHandler.getAllTables(getName()), tables.keySet());
    }

    @Override
    public void addEntityWrapper(String name) throws SQLException {
        Table table = tables.get(name);
        if (table == null) {
            // Add new 
            TableEntity tableEntity = tapSchemaHandler.getNewTable(getName(), name);
            table = new Table(tapSchemaHandler, getName(), name, tableEntity);
            tables.put(name, table);
            table.setStatus(Status.ADDED_NOT_PERSISTED);
        } else {
            // Undo remove
            table.setStatus(Status.ADDED_PERSISTED);
        }
        selectedTable = name;

        tapSchemaHandler.addTable(getName(), table.getEntity());
    }

    @Override
    public void removeEntityWrapper(String name) {
        String previousTable = null;
        boolean changeSelection = false;
        TableEntity tableEntity = null;
        for (Table table : tables.values()) {
            if (table.getName().equals(name)) {
                if (table.getStatus() == Status.ADDED_NOT_PERSISTED) {
                    tables.remove(name);
                    changeSelection = true;
                } else {
                    table.setStatus(Status.TO_REMOVE);
                }
                tableEntity = table.getEntity();
                break;
            }
            previousTable = table.getName();
        }

        if (changeSelection) {
            if (tables.isEmpty()) {
                selectedTable = null;
            } else {
                selectedTable = previousTable == null ? tables.keySet().iterator().next() : previousTable;
            }
        }

        tapSchemaHandler.removeTable(getEntity(), tableEntity);
    }

    @Override
    public Table getEntity(String name) {
        return tables.get(name);
    }

    @Override
    protected void setPersisted() {
        super.setPersisted();
        Iterator<Table> iterator = tables.values().iterator();
        while (iterator.hasNext()) {
            Table table = iterator.next();
            if (table.getStatus() == Status.TO_REMOVE) {
                iterator.remove();
            } else {
                table.setPersisted();
            }
        }
    }

    @Override
    public boolean hasChildren() {
        return !tables.isEmpty();
    }

    @Override
    public List<Table> getAllEntityWrappers() {
        return new ArrayList(tables.values());
    }

    @Override
    public void selectEntity(String entityName) {
        selectedTable = entityName;
        Table table = tables.get(entityName);
        List<String> columns = table.getEntitiesNames();
        if (!columns.isEmpty()) {
            table.selectEntity(columns.get(0));
        }
    }

    @Override
    public Table getSelectedEntity() {
        return selectedTable == null ? null : tables.get(selectedTable);
    }

    public String getUtype() {
        return getValue(UTYPE);
    }

    public void setUtype(String utype) {
        setValue(UTYPE, utype);
    }

    public String getDescription() {
        return getValue(DESCRIPTION);
    }

    public void setDescription(String description) {
        setValue(DESCRIPTION, description);
    }
}
