package it.inaf.oats.ia2.tapschemamanager.datalayer;

import com.mysql.jdbc.jdbc2.optional.MysqlDataSource;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.SQLException;
import javax.sql.DataSource;
import org.postgresql.ds.PGPoolingDataSource;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public class DBWrapper implements Serializable {

    private static final long serialVersionUID = 1721030677924066695L;

    // Same credentials
    private Credentials credentials;

    // Separated credentials
    private Credentials sourceCredentials;
    private Credentials tapSchemaCredentials;

    private final DataSourcesWrapper dataSources;

    private DBWrapper() {
        dataSources = new DataSourcesWrapper();
    }

    /**
     * Constructor to use if the source schema credentials and the TAP_SCHEMA
     * credentials are the same.
     */
    public DBWrapper(Credentials credentials) {
        this();
        this.credentials = credentials;
    }

    /**
     * Constructor to use if the source schema credentials are different from
     * the TAP_SCHEMA credentials.
     */
    public DBWrapper(Credentials sourceCredentials, Credentials tapSchemaCredentials) {
        this();
        this.sourceCredentials = sourceCredentials;
        this.tapSchemaCredentials = tapSchemaCredentials;
    }

    public Connection getSourceConnection() throws SQLException {
        return dataSources.getSourceDataSource().getConnection();
    }

    public Connection getTapSchemaConnection() throws SQLException {
        return dataSources.getTapSchemaDataSource().getConnection();
    }

    public Credentials getSourceCredentials() {
        if (credentials != null) {
            return credentials;
        }
        return sourceCredentials;
    }

    public Credentials getTapSchemaCredentials() {
        if (credentials != null) {
            return credentials;
        }
        return tapSchemaCredentials;
    }

    public DatabaseType getSourceDatabaseType() {
        return getSourceCredentials().getDatabaseType();
    }

    public DatabaseType getTapSchemaDatabaseType() {
        return getTapSchemaCredentials().getDatabaseType();
    }

    public void testConnections() throws SQLException {
        Connection connection;
        if (credentials != null) {
            connection = dataSources.getSourceDataSource().getConnection();
            connection.close();
        } else {
            connection = dataSources.getSourceDataSource().getConnection();
            connection.close();
            connection = dataSources.getTapSchemaDataSource().getConnection();
            connection.close();
        }
    }

    private class DataSourcesWrapper implements Serializable {

        private static final long serialVersionUID = -7025255003212206748L;

        private transient DataSource dataSource;
        private transient DataSource sourceDataSource;
        private transient DataSource tapSchemaDataSource;

        public DataSource getTapSchemaDataSource() {
            if (credentials != null) {
                if (dataSource == null) {
                    dataSource = createDataSource(credentials);
                }
                return dataSource;
            }
            if (tapSchemaDataSource == null) {
                tapSchemaDataSource = createDataSource(tapSchemaCredentials);
            }
            return tapSchemaDataSource;
        }

        public DataSource getSourceDataSource() {
            if (credentials != null) {
                if (dataSource == null) {
                    dataSource = createDataSource(credentials);
                }
                return dataSource;
            }
            if (sourceDataSource == null) {
                sourceDataSource = createDataSource(sourceCredentials);
            }
            return sourceDataSource;
        }

        private DataSource createDataSource(Credentials credentials) {
            if (credentials.getDatabaseType() == DatabaseType.MYSQL) {

                MysqlDataSource ds = new MysqlDataSource();

                ds.setServerName(credentials.getHostname());
                ds.setPortNumber(credentials.getPort());
                ds.setUser(credentials.getUsername());
                ds.setPassword(credentials.getPassword());

                return ds;
            } else if (credentials.getDatabaseType() == DatabaseType.POSTGRES) {

                PGPoolingDataSource ds = new PGPoolingDataSource();

                ds.setServerName(credentials.getHostname());
                ds.setPortNumber(credentials.getPort());
                ds.setUser(credentials.getUsername());
                ds.setPassword(credentials.getPassword());

                return ds;
            }

            throw new UnsupportedOperationException(credentials.getDatabaseType() + " not supported yet.");
        }
    }
}
